﻿//  ------------------------------------------------------
//  Copyright (c) Andrei Marukovich. All rights reserved.
//  This source code is a part of TaggedFrog software
//  http://lunarfrog.com/
//  ------------------------------------------------------
using System;
using System.IO;
using System.Xml.Serialization;
using DSOleFile;

namespace TaggedFrog.Extensions.Meta
{
   public class MetaExtension : ITaggingExtension
   {
      /// <summary>
      /// current translation
      /// </summary>
      private MetaTranslation m_current;

      #region ITaggingExtension Members

      public void Init(string executingPath, string language)
      {
         m_current = MetaTranslation.Load(String.Format("{0}\\translation\\meta_{1}.xml", executingPath, language));
      }

      public string[] GetVerbs()
      {
         return new[] { m_current.VerbName, m_current.VerbMeta};
      }

      public string GetKeywords(string verb, string path, bool isURL)
      {
         if (isURL)
         {
            return String.Empty;
         }

         string result = String.Empty;
         if (verb.Equals(m_current.VerbName))
         {
            try
            {
               string name = Path.GetFileNameWithoutExtension(path);
               string[] parts = name.Split(new[] { ' ', '_' }, StringSplitOptions.RemoveEmptyEntries);
               if (parts.Length > 1)
               {
                  result = String.Join(", ", parts);
               }
            }
            catch
            {
               // There is no reason to inform user, skip error
            }
         }
         else if (verb.Equals(m_current.VerbMeta))
         {
            try
            {
               PropertyReaderClass propReader = new PropertyReaderClass();
               DocumentProperties props = propReader.GetDocumentProperties(path);

               if (props.Keywords == null)
               {
                  result = String.Empty;
               }
               else
               { 
                  string[] parts = props.Keywords.Split(new[] { ' ', ',', ';' }, 
                                                        StringSplitOptions.RemoveEmptyEntries);
                  if (parts.Length == 1)
                  {
                     result = parts[0];
                  }
                  else if (parts.Length > 1)
                  {
                     result = String.Join(", ", parts);
                  }
               }

               props = null;
               propReader = null;
            }
            catch
            {
               // COM error, there is no reason to inform user, skip error
            }
         }

         return result;
      }

      public string GetHint(string verb)
      {
         return String.Empty;
      }

      public bool IsSetupNeeded(string verb)
      {
         return false;
      }

      public bool ShowSetup(string verb)
      {
         return false;
      }
      #endregion
   }

   public class MetaTranslation
   {
      public string VerbName = "Keywords from file name";
      public string VerbMeta = "Keywords from file meta information";

      public static MetaTranslation Load(string path)
      {
         MetaTranslation obj;
         try
         {
            using (FileStream fs =
               new FileStream(path, FileMode.Open))
            {
               XmlSerializer xs = new XmlSerializer(typeof(MetaTranslation));
               obj = (MetaTranslation)xs.Deserialize(fs);
            }
         }
         catch
         {
            obj = new MetaTranslation();
         }

         return obj;
      }
   }
}